<?php
// ---------------------------------------------------------------------
// BRIDGE: Coppermine 1.4.14 <-> phpBB 3.0.0,
// written by Stephen from team-andro.com
// Version history:
// - v0.9.0, final beta at 02.01.2008
// - v0.9.1, fixed bug "dashes in database name" at 03.01.2008
// - v0.9.2, fixed bug "session refresh" at 05.01.2008
// - v0.9.3, fixed bug "no session update" at 10.01.2008
// see file "installation instructions v0.9.3.txt"
// ---------------------------------------------------------------------

/*************************
  Coppermine Photo Gallery
  ************************
  Copyright (c) 2003-2007 Coppermine Dev Team
  v1.1 originally written by Gregory DEMAR

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  ********************************************
  Coppermine version: 1.4.14
**********************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

// Switch that allows overriding the bridge manager with hard-coded values
define('USE_BRIDGEMGR', 1);

// ------------------------------------------------------
// Stephen, 28.12.2007
// -> Define basic group constants
// Admin = 1, User = 2, Guest = 3, Bot = 4
// ------------------------------------------------------
define('ADMIN_GRP', 1);
define('USER_GRP', 2);
define('GUEST_GRP', 3);
define('BOT_GRP', 4);
// ------------------------------------------------------

// include base class
require_once 'bridge/udb_base.inc.php';

//-----------------------------------------------------------------------------
/** bridge class for phpbb3 (RC3)
*/
//-----------------------------------------------------------------------------
class phpbb3_udb extends core_udb
{
	//---------------------------------------------------------------------------
	/** construct bridge
	*/
	function phpbb3_udb()
	{
		global $BRIDGE;

		// set some vars from the bridgemgr
		$this->boardurl = $BRIDGE['full_forum_url'];
		$this->use_post_based_groups = $BRIDGE['use_post_based_groups'];

		// call phpbb3 config.php -> this sets database variables
		require_once($BRIDGE['relative_path_to_config_file'] . 'config.php');

		// Database connection settings from phpbb3's config file
		$this->db = array(
		  'name' => $dbname,
		  'host' => $dbhost ? $dbhost : 'localhost',
		  'user' => $dbuser,
		  'password' => $dbpasswd,
		  'prefix' => $table_prefix
		);

		// Combination of database name and table prefix
		$this->pref = '`' . $this->db['name'] . '`.' . $this->db['prefix'];

		// phpbb3 uses multigroups
		$this->multigroups = 1;

		// Board table names
		$this->table = array(
		  'users' => 'users',
		  'groups' => 'groups',
		  'sessions' => 'sessions',
		  'usergroups' => 'user_group',
		  'sessionskeys' => 'sessions_keys'
		);

		// Derived full table names
		$this->usertable = $this->pref . $this->table['users'];
		$this->groupstable =  $this->pref . $this->table['groups'];
		$this->sessionstable =  $this->pref . $this->table['sessions'];
		$this->usergroupstable = $this->pref . $this->table['usergroups'];
		$this->sessionskeystable = $this->pref . $this->table['sessionskeys'];

		// Table field names
		$this->field = array(
		  'username' => 'username', // name of 'username' field in users table
		  'user_id' => 'user_id', // name of 'id' field in users table
		  'password' => 'user_password', // name of 'password' field in users table
		  'email' => 'user_email', // name of 'email' field in users table
		  'regdate' => 'user_regdate', // name of 'registered' field in users table
		  'active' => 'user_active', // is user account active?
		  'lastvisit' => 'user_lastvisit', // name of 'location' field in users table
		  'location' => 'user_from', // name of 'location' field in users table
		  'website' => 'user_website', // name of 'website' field in users table
		  'usertbl_group_id' => 'group_id', // name of 'group id' field in users table
		  'grouptbl_group_id' => 'group_id', // name of 'group id' field in groups table
		  'grouptbl_group_name' => 'group_name' // name of 'group name' field in groups table
		);

		// Pages to redirect to
		$this->page = array(
		  'register' => '/ucp.php?mode=register',
		  'editusers' => '/memberlist.php',
		  'edituserprofile' => "/memberlist.php?mode=viewprofile&u=",
		);

		// Group ids - admin and guest only.
		$this->admingroups = array(ADMIN_GRP);
		$this->guestgroup = GUEST_GRP;
		$this->botgroup = BOT_GRP;

		// Use a special function to collect groups for cpg groups table
		$this->group_overrride = 1;

		// get path for redirection
		$this->redirect_url = relative_paths(basename($_SERVER['PHP_SELF']), $BRIDGE['relative_path_to_config_file'].'config.php');

		// Connect to db
		$this->connect();

		// --------------------------------------------------
		// Stephen, 28.12.2007
		// After connecting we can access phpbb3 database
		// and get the cookie_name
		// --------------------------------------------------
		$sql =
		"SELECT
			config_name,
			config_value
		FROM
			" . $this->pref . "config" . "
		WHERE
			config_name IN ('cookie_name', 'session_length', 'allow_autologin', 'max_autologin_time')";

		$result = cpg_db_query($sql, $this->link_id);
		if ( !mysql_num_rows($result) )
		{
			cpg_die(ERROR, "DB error: file %s, line %s", __FILE__, __LINE__);
		}
		else
		{
			while ( $row = mysql_fetch_array($result) )
			{
				$this->$row['config_name'] = $row['config_value'];
			}
		}
		mysql_free_result($result);
	}

	//---------------------------------------------------------------------------
	/** get group definitions from phpbb3
	*/
	function collect_groups()
	{
		// -----------------------------------------------
		// Stephen, 28.12.2007
		// use_post_based_groups => only basic groups
		// else => basic groups and phpBB3 groups
		// -----------------------------------------------
		$udb_groups = array(
			ADMIN_GRP => 'CPG Admin',
			USER_GRP => 'CPG Registered Users',
			GUEST_GRP => 'CPG Guests',
			BOT_GRP => 'CPG Bots');

		if ( !$this->use_post_based_groups )
		{
			return $udb_groups;
		}

		// select all groups except guests and bots
		$sql =
		"SELECT
			group_id, group_name
		FROM
			". $this->pref . "groups
		WHERE
			group_name != 'BOTS' AND
			group_name != 'GUESTS'
		ORDER BY group_id";

		$result = cpg_db_query($sql, $this->link_id);
		while ( $row = mysql_fetch_assoc($result) )
		{
		    // offset all phpbb3 defined groups by 100
		    $udb_groups[$row[$this->field['grouptbl_group_id']] + 100] = utf8_encode($row[$this->field['grouptbl_group_name']]);
		}
		mysql_free_result($result);

		return $udb_groups;
	}


	// ---------------------------------------------------------------------------
	// Cookie extraction phpBB3-like ...
	// ---------------------------------------------------------------------------
	function session_extraction()
	{
	  	// only check cookie - coppermine doesn't work with url sid
	  	if ( isset($_COOKIE[$this->cookie_name . '_sid']) )
		{
			// extract session id
			$this->sid = $_COOKIE[$this->cookie_name . '_sid'];

			if ( $this->sid != "" )
			{
				// get session data, user_type != 2 excludes guests and bots
				$sql =
				"SELECT
					u.*, s.*
				FROM
					" . $this->usertable . " u,
					" . $this->sessionstable . " s
				WHERE
					u.user_id = s.session_user_id AND
					u.user_type != 2 AND
					u.user_type != 1 AND
					s.session_id = " . quote_smart($this->sid, $this->link_id);

				$result = cpg_db_query($sql, $this->link_id);
	     		$this->session_data = mysql_fetch_array($result);
	       		mysql_free_result($result);

	       		// session exists
	       		if ( $this->session_data['user_id'] )
	       		{
	       			$session_expired = false;

	       			if (!$this->session_data['session_autologin'])
					{
						if ($this->session_data['session_time'] < time() - ($this->session_length + 60))
						{
							$session_expired = true;
						}
					}
					else if (!$this->allow_autologin || ($this->max_autologin_time && $this->session_data['session_time'] < time() - (86400 * (int) $this->max_autologin_time) + 60))
					{
						$session_expired = true;
					}

					return $session_expired ? false : true;
	       		}
			}
		}

		return false;
	}

	// -------------------------------------------------------------------------------
	// Stephen, 28.12.2007
	// authenticate -> read cookie; check whether admin, user, guest or bot.
	// Then pull user infos
	// -------------------------------------------------------------------------------
	function authenticate()
	{
		global $USER_DATA, $BRIDGE;

		// registered check
		if ( $this->session_extraction() )
		{
			$this->is_guest = false;
			$this->is_bot = false;
			$this->is_registered = true;

			// admin check
			$this->is_admin = false;

			// check assignment through group and role
			$sql =
			"SELECT
				*
			FROM
				" . $this->pref . "user_group ug,
				" . $this->pref . "acl_groups ag,
				" . $this->pref . "acl_roles_data ard,
				" . $this->pref . "acl_options ao
			WHERE
				ug.user_id = " . $this->session_data['user_id'] . " AND
				ug.group_id = ag.group_id AND
				ag.auth_role_id = ard.role_id AND
				ard.auth_setting = 1 AND
				ard.auth_option_id = ao.auth_option_id AND
				(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')";

			$result = cpg_db_query($sql, $this->link_id);
			if ( mysql_num_rows($result) )
			{
				$this->is_admin = true;
			}
			mysql_free_result($result);

			if ( !$this->is_admin )
			{
				// check assignment through group
				$sql =
				"SELECT
					*
				FROM
					" . $this->pref . "user_group ug,
					" . $this->pref . "acl_groups ag,
					" . $this->pref . "acl_options ao
				WHERE
					ug.user_id = " . $this->session_data['user_id'] . " AND
					ug.group_id = ag.group_id AND
					ag.auth_option_id = ao.auth_option_id AND
					ag.auth_setting = 1 AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')";

				$result = cpg_db_query($sql, $this->link_id);
				if ( mysql_num_rows($result) )
				{
					$this->is_admin = true;
				}
				mysql_free_result($result);
			}

			if ( !$this->is_admin )
			{
				// check assignment through role
				$sql =
				"SELECT
					*
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_roles_data ard,
					" . $this->pref . "acl_options ao
				WHERE
					au.user_id = " . $this->session_data['user_id'] . " AND
					au.auth_role_id = ard.role_id AND
					ard.auth_setting = 1 AND
					ard.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')";

				$result = cpg_db_query($sql, $this->link_id);
				if ( mysql_num_rows($result) )
				{
					$this->is_admin = true;
				}
				mysql_free_result($result);
			}

			if ( !$this->is_admin )
			{
				// directly assigned to user
				"SELECT
					*
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_options ao
				WHERE
					au.user_id = " . $this->session_data['user_id'] . " AND
					au.auth_setting = 1 AND
					au.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')";

				$result = cpg_db_query($sql, $this->link_id);
				if ( mysql_num_rows($result) )
				{
					$this->is_admin = true;
				}
				mysql_free_result($result);
			}
		}
		else
		{
			$this->is_registered = false;
			$this->is_admin = false;

			// check bot
			$this->is_guest = true;
			$this->is_bot = false;

			if ( $BRIDGE['bot_login'] )
			{
				$sql =
				"SELECT
					*
				FROM
					" . $this->pref . "bots
				WHERE
					INSTR(" . quote_smart($_SERVER['HTTP_USER_AGENT'], $this->link_id) . ", bot_agent) > 0 AND
					(bot_ip = '' OR bot_ip = " . quote_smart($_SERVER['REMOTE_ADDR'], $this->link_id) . ")";

				$result = cpg_db_query($sql, $this->link_id);
				if ( $row = mysql_fetch_array($result) )
				{
					$this->is_guest = false;
					$this->is_bot = true;
					$this->bot_data = $row;
				}
				mysql_free_result($result);
			}
		}

		// if we reach here, we know whether guest, user, admin or bot
		if ($this->is_registered)
		{
			$this->load_user_data($this->session_data);

			$user_group_set = '(' . implode(',', $USER_DATA['groups']) . ')';
			$USER_DATA = array_merge(
				$USER_DATA,
				$this->get_user_data($this->is_admin ? ADMIN_GRP : USER_GRP, $USER_DATA['groups'], USER_GRP));

			// admins always get admin access
			$USER_DATA['has_admin_access'] = $this->is_admin ? 1 : $USER_DATA['has_admin_access'];
			$USER_DATA['can_see_all_albums'] = $this->is_admin ? 1 : $USER_DATA['can_see_all_albums'];

			$this->session_update();
		}
		else
		{
			if ($this->is_bot)
			{
				$this->load_bot_data($this->bot_data);

        		$user_group_set = "(" . BOT_GRP . ")";
				$USER_DATA = array_merge($USER_DATA, $this->get_user_data(BOT_GRP, array(BOT_GRP), BOT_GRP));
			}
			else
			{
				$this->load_guest_data();

				$user_group_set = "(" . GUEST_GRP . ")";
				$USER_DATA = array_merge($USER_DATA, $this->get_user_data(GUEST_GRP, array(GUEST_GRP), GUEST_GRP));
			}

			// bots and guests never get admin access
			$USER_DATA['has_admin_access'] = 0;
			$USER_DATA['can_see_all_albums'] = 0;
		}


		// ??? avoids a template error
		// so, keep the code
		if (!$USER_DATA['user_id'])
			$USER_DATA['can_create_albums'] = 0;

		// For error checking
		$CONFIG['TABLE_USERS'] = '**ERROR**';

		define('USER_ID', $USER_DATA['user_id']);
        define('USER_NAME', addslashes($USER_DATA['user_name']));
        define('USER_GROUP', $USER_DATA['group_name']);
        define('USER_GROUP_SET', $user_group_set);
        define('USER_IS_ADMIN', $USER_DATA['has_admin_access']);
        define('USER_CAN_SEND_ECARDS', (int)$USER_DATA['can_send_ecards']);
        define('USER_CAN_RATE_PICTURES', (int)$USER_DATA['can_rate_pictures']);
        define('USER_CAN_POST_COMMENTS', (int)$USER_DATA['can_post_comments']);
        define('USER_CAN_UPLOAD_PICTURES', (int)$USER_DATA['can_upload_pictures']);
        define('USER_CAN_CREATE_ALBUMS', (int)$USER_DATA['can_create_albums']);
        define('USER_UPLOAD_FORM', (int)$USER_DATA['upload_form_config']);
        define('CUSTOMIZE_UPLOAD_FORM', (int)$USER_DATA['custom_user_upload']);
        define('NUM_FILE_BOXES', (int)$USER_DATA['num_file_upload']);
        define('NUM_URI_BOXES', (int)$USER_DATA['num_URI_upload']);
	}


	// -------------------------------------------------------------------------------
	// Stephen, 28.12.2007
	// Separate load functions for users (incl. admins), bots and guests
	// -------------------------------------------------------------------------------
	function load_user_data($session_data)
	{
		global $USER_DATA;

		$USER_DATA['user_id'] = $session_data['user_id'];
        $USER_DATA['user_name'] = utf8_encode($session_data['username']);

        // get main group name
        $sql =
        "SELECT
        	group_name
        FROM
        	" . $this->pref . "groups
        WHERE
        	group_id = " . $session_data['group_id'];

        $result = cpg_db_query($sql, $this->link_id);
		if ( !$row = mysql_fetch_array($result) )
		{
			cpg_die(ERROR, "DB error: file %s, line %s", __FILE__, __LINE__);
		}
		else
		{
			$USER_DATA['group_name'] = utf8_encode($row['group_name']);
		}
        mysql_free_result($result);

		$USER_DATA['groups'] = $this->get_groups($session_data);
	}

	function load_guest_data()
    {
		global $USER_DATA;

		$USER_DATA['user_id'] = 0;
		$USER_DATA['user_name'] = 'Guest';

		$USER_DATA['group_name'] = 'CPG Guests';
		$USER_DATA['groups'] = array(GUEST_GRP);
        $USER_DATA['group_quota'] = 1;
        $USER_DATA['can_rate_pictures'] = 0;
		$USER_DATA['can_send_ecards'] = 0;
		$USER_DATA['can_post_comments'] = 0;
		$USER_DATA['can_upload_pictures'] = 0;
		$USER_DATA['can_create_albums'] = 0;
		$USER_DATA['pub_upl_need_approval'] = 1;
		$USER_DATA['priv_upl_need_approval'] = 1;
		$USER_DATA['upload_form_config'] = 0;
		$USER_DATA['num_file_upload'] = 0;
		$USER_DATA['num_URI_upload'] = 0;
		$USER_DATA['custom_user_upload'] = 0;
	}

	function load_bot_data($bot_data)
    {
		global $USER_DATA;

		$USER_DATA['user_id'] = $bot_data['user_id'];
        $USER_DATA['user_name'] = utf8_encode($bot_data['bot_name']);

		$USER_DATA['group_name'] = 'CPG Bots';
		$USER_DATA['groups'] = array(BOT_GRP);
        $USER_DATA['group_quota'] = 1;
        $USER_DATA['can_rate_pictures'] = 0;
		$USER_DATA['can_send_ecards'] = 0;
		$USER_DATA['can_post_comments'] = 0;
		$USER_DATA['can_upload_pictures'] = 0;
		$USER_DATA['can_create_albums'] = 0;
		$USER_DATA['pub_upl_need_approval'] = 1;
		$USER_DATA['priv_upl_need_approval'] = 1;
		$USER_DATA['upload_form_config'] = 0;
		$USER_DATA['num_file_upload'] = 0;
		$USER_DATA['num_URI_upload'] = 0;
		$USER_DATA['custom_user_upload'] = 0;
	}


	//---------------------------------------------------------------------------
	/** get users groups
	*/
	function get_groups($session_data)
	{
		global $BRIDGE, $USER_DATA;

		$data = array();
		$user_id = $USER_DATA['user_id'];

		if ( $this->is_registered )
		{
			$data[] = USER_GRP;
		}

		if ( $this->is_admin )
		{
			$data[] = ADMIN_GRP;
		}

		if ( $this->use_post_based_groups )
		{
			// select all groups of user
			$sql =
			"SELECT
				ug.group_id AS group_id,
				u.group_id AS maingroup
			FROM
				" . $this->pref . "user_group ug,
				" . $this->pref . "users u
			WHERE
				u.user_id = ug.user_id AND
				u.user_id = $user_id";

			$result = cpg_db_query($sql, $this->link_id);

			while ($row = mysql_fetch_assoc($result))
			{
				$data[] = 100 + $row['group_id'];
			}
			mysql_free_result($result);
		}

		return $data;
	}

	//---------------------------------------------------------------------------
	/** definition of actions required to convert a password from user database form to cookie form
	*/
	function udb_hash_db($password)
	{
		return $password; // unused
	}

	//---------------------------------------------------------------------------
	/** redirect to login page
	*/
	function login_page()
	{
		$this->redirect("/ucp.php?mode=login&redirect={$this->redirect_url}");
	}

	//---------------------------------------------------------------------------
	/** redirect to logout page
	*/
	function logout_page()
	{
		$this->redirect("/ucp.php?mode=logout&sid={$this->sid}&redirect={$this->redirect_url}");
	}

	//---------------------------------------------------------------------------
	/** redirect to user list (available for admins only)
	*/
	function view_users()
	{
		// uncomment the next line if you want to see phpbb3's user list instead of coppermine's one
		// $this->redirect("memberlist.php?sid={$this->sid}");
	}

	//---------------------------------------------------------------------------
	/** redirect to user's profile
	*/
	function view_profile($uid)
	{
		global $USER_DATA;

		if ( $uid != '' )
		{
			$this->redirect("/memberlist.php?mode=viewprofile&sid={$this->sid}&u={$uid}");
		}
		elseif ( $_GET['op'] == 'edit_profile' )
		{
			$this->redirect("/memberlist.php?mode=viewprofile&sid={$this->sid}&u={$USER_DATA['user_id']}");
		}

	}

	// Edit user profile
    function edit_profile($uid)
    {
        $this->redirect($this->page['edituserprofile']);
    }

	// Group table synchronisation, called when (re)installing the bridge
	function synchronize_groups()
	{
        global $CONFIG ;

        $udb_groups = $this->collect_groups();

        // init of group_id_list, this is needed for delete later
       	$group_id_list = "(";

       	// loop over groups, update or insert
		foreach($udb_groups as $group_id => $group_name)
        {
        	//build group_id_list
        	if ( $group_id_list == "(" )
        	{
        		$group_id_list .= $group_id;
        	}
        	else
        	{
        		$group_id_list .= ", " . $group_id;
        	}

        	$sql = "SELECT * FROM " . $CONFIG['TABLE_USERGROUPS'] . " WHERE group_id = $group_id";

        	$result = cpg_db_query($sql);
			$row = mysql_fetch_array($result);
			mysql_free_result($result);

			if ( $row && $group_id == ADMIN_GRP )
			{
				// look whether we have to update the group
				$sql =
				"UPDATE
					" . $CONFIG['TABLE_USERGROUPS'] . "
				SET
					has_admin_access = 1
				WHERE
					group_id = " . $group_id;

				cpg_db_query($sql);
			}
			elseif ( !$row || $row['group_name'] != $group_name )
			{
				// delete group and insert if name change
				if ( $row['group_name'] != $group_name )
				{
					$sql =
					"DELETE FROM
						" . $CONFIG['TABLE_USERGROUPS'] . "
					WHERE
						group_id = $group_id";

					cpg_db_query($sql);
				}

				//no group or name change, so let's insert it
				$sql =
				"INSERT INTO " . $CONFIG['TABLE_USERGROUPS'] . "
					(group_id,
					group_name,
					group_quota,
					has_admin_access,
					can_rate_pictures,
					can_send_ecards,
					can_post_comments,
					can_upload_pictures,
					can_create_albums,
					pub_upl_need_approval,
					priv_upl_need_approval)
				VALUES
					($group_id,
					" . quote_smart($group_name, $this->link_id) . ",
					0,
					" . (($group_id == ADMIN_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP || $group_id == USER_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP || $group_id == USER_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP || $group_id == USER_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP || $group_id == USER_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP) ? 1 : 0) . ",
					" . (($group_id == ADMIN_GRP || $group_id == USER_GRP) ? 0 : 1) . ",
					" . (($group_id == ADMIN_GRP) ? 0 : 1) . ")";

				cpg_db_query($sql);
			}
        }
        $group_id_list .= ")";

        // delete all groups no more needed
        $sql =
        "DELETE FROM
        	" . $CONFIG['TABLE_USERGROUPS'] . "
        WHERE
        	group_id NOT IN $group_id_list";

        cpg_db_query($sql);
	}

	// --------------------------------------------------------------------------
	// get full user list -> is needed for userlist in admin mode
	// --------------------------------------------------------------------------
	function get_users($options = array())
	{
		global $CONFIG;

		// Copy UDB fields and config variables (just to make it easier to read)
		$f =& $this->field;
		$C =& $CONFIG;

		// Sort codes - global this in usermgr.php in 1.5
		$sort_codes = array('name_a' => 'user_name ASC',
		                    'name_d' => 'user_name DESC',
		                    'group_a' => 'group_name ASC',
		                    'group_d' => 'group_name DESC',
		                    'reg_a' => 'user_regdate ASC',
		                    'reg_d' => 'user_regdate DESC',
		                    'pic_a' => 'pic_count ASC',
		                    'pic_d' => 'pic_count DESC',
		                    'disku_a' => 'disk_usage ASC',
		                    'disku_d' => 'disk_usage DESC',
		                    'lv_a' => 'user_lastvisit ASC',
		                    'lv_d' => 'user_lastvisit DESC',
		                   );

		$sql = "SELECT group_id, group_name, group_quota FROM {$C['TABLE_USERGROUPS']}";
		$result = cpg_db_query($sql);

		$groups = $quotas = array();

		while ($row = mysql_fetch_assoc($result))
		{
			$groups[$row['group_id']] = utf8_encode($row['group_name']);
			$quotas[$row['group_id']] = $row['group_quota'];
		}

		mysql_free_result($result);

		//if (in_array($options['sort'], array('group_a', 'group_d', 'pic_a', 'pic_d', 'disku_a', 'disku_d')))
		if (in_array($options['sort'], array('pic_a', 'pic_d', 'disku_a', 'disku_d')))
		{
			$sort = '';
			list($this->sortfield, $this->sortdir) = explode(' ', $sort_codes[$options['sort']]);
			$this->adv_sort = true;
		}
		else
		{
			$sort = "ORDER BY " . $sort_codes[$options['sort']];
			$this->adv_sort = false;
		}

    	// Build WHERE clause, if this is a username search
        if ($options['search'])
        {
            $options['search'] = 'AND u.' . $f['username'] . " LIKE '" . $options['search'] . "' ";
        }

		// Main array to hold our user data
		$userlist = array();

		// These sorting methods need the cpg pics table, do that first
		if (in_array($options['sort'], array('pic_a', 'pic_d', 'disku_a', 'disku_d')))
		{
     		$sql =
     		"SELECT
     			owner_id,
     			COUNT(pid) as pic_count,
     			ROUND(SUM(total_filesize)/1024) as disk_usage
     		FROM
     			{$C['TABLE_PICTURES']}
     		WHERE
     			owner_id <> 0
     		GROUP BY owner_id ORDER BY {$sort_codes[$options['sort']]}
     		LIMIT {$options['lower_limit']}, {$options['users_per_page']}";

			$result = cpg_db_query($sql);

			// If no records, return empty value
			if (!mysql_num_rows($result))
			{
				return array();
			}

			while ($row = mysql_fetch_assoc($result))
			{
				$userlist[$row['owner_id']] = $row;
			}
			mysql_free_result($result);

			$user_list_string = implode(', ', array_keys($userlist));

			// nice sql statement :-)
			// needed to check for gallery admin and bots
			$sql =
			"SELECT
				u.user_id,
				CASE
					WHEN ISNULL(adm.user_id) AND ISNULL(b.user_id) THEN 2
					WHEN ISNULL(b.user_id) THEN 1
					ELSE 4
				END AS group_id,
				CASE
					WHEN ISNULL(adm.user_id) AND ISNULL(b.user_id) THEN 'CPG Registered Users'
					WHEN ISNULL(b.user_id) THEN 'CPG Admin'
					ELSE 'CPG Bots'
				END AS group_name,
				u.username as user_name,
				user_email as user_email,
				user_regdate as user_regdate,
				user_lastvisit as user_lastvisit
			FROM
				" . $this->pref . "users u
			LEFT JOIN
				(SELECT
					ug.user_id
				FROM
					" . $this->pref . "user_group ug,
					" . $this->pref . "acl_groups ag,
					" . $this->pref . "acl_roles_data ard,
					" . $this->pref . "acl_options ao
				WHERE
					ug.group_id = ag.group_id AND
					ag.auth_role_id = ard.role_id AND
					ard.auth_setting = 1 AND
					ard.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					ug.user_id
				FROM
					" . $this->pref . "user_group ug,
					" . $this->pref . "acl_groups ag,
					" . $this->pref . "acl_options ao
				WHERE
					ug.group_id = ag.group_id AND
					ag.auth_option_id = ao.auth_option_id AND
					ag.auth_setting = 1 AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					au.user_id
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_roles_data ard,
					" . $this->pref . "acl_options ao
				WHERE
					au.auth_role_id = ard.role_id AND
					ard.auth_setting = 1 AND
					ard.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					au.user_id
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_options ao
				WHERE
					au.auth_setting = 1 AND
					au.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')) adm
			ON
				u.user_id = adm.user_id
			LEFT JOIN
				" . $this->pref . "bots b
			ON
				u.user_id = b.user_id
			WHERE
				u.user_id > 1 AND
				u.user_id IN ($user_list_string)";

			$result = cpg_db_query($sql, $this->link_id);

			// If no records, return empty value
			if (!mysql_num_rows($result))
			{
				return array();
			}

			while ($row = mysql_fetch_assoc($result))
			{
				$userlist[$row['user_id']] = array_merge($userlist[$row['user_id']], $row);
			}
			mysql_free_result($result);
		}
		else
		{
			// let's do a preselect for performance
			$preselect = false;
			if ( !in_array($options['sort'], array('group_a', 'group_d')) )
			{
				$preselect = true;
				$temp_list = array();

				$sql =
				"SELECT
					u.user_id,
					u.username AS user_name
				FROM
					" . $this->pref . "users u
				WHERE
					u.user_id > 1
					" . $options['search'] . $sort . "
				LIMIT " . $options['lower_limit'] . "," . $options['users_per_page'];

				$result = cpg_db_query($sql, $this->link_id);

				// If no records, return empty value
				if (!mysql_num_rows($result))
				{
					return array();
				}

				while ($row = mysql_fetch_assoc($result))
				{
					$temp_list[] = $row['user_id'];
				}
				mysql_free_result($result);

				$temp_list_string = implode(', ', $temp_list);
			}

			if ( $preselect )
			{
				$sql_footer =
				"WHERE
					u.user_id IN (" . $temp_list_string . ")" . $sort ;
			}
			else
			{
				$sql_footer =
				"WHERE
					u.user_id > 1
					" . $options['search'] . $sort . "
				LIMIT " . $options['lower_limit'] . "," . $options['users_per_page'];
			}


			$sql =
			"SELECT
				u.user_id,
				CASE
					WHEN ISNULL(adm.user_id) AND ISNULL(b.user_id) THEN 2
					WHEN ISNULL(b.user_id) THEN 1
					ELSE 4
				END AS group_id,
				CASE
					WHEN ISNULL(adm.user_id) AND ISNULL(b.user_id) THEN 'CPG Registered Users'
					WHEN ISNULL(b.user_id) THEN 'CPG Admin'
					ELSE 'CPG Bots'
				END AS group_name,
				u.username as user_name,
				user_email as user_email,
				user_regdate as user_regdate,
				user_lastvisit as user_lastvisit,
				0 as pic_count
			FROM
				" . $this->pref . "users u
			LEFT JOIN
				(SELECT
					ug.user_id
				FROM
					" . $this->pref . "user_group ug,
					" . $this->pref . "acl_groups ag,
					" . $this->pref . "acl_roles_data ard,
					" . $this->pref . "acl_options ao
				WHERE
					ug.group_id = ag.group_id AND
					ag.auth_role_id = ard.role_id AND
					ard.auth_setting = 1 AND
					ard.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					ug.user_id
				FROM
					" . $this->pref . "user_group ug,
					" . $this->pref . "acl_groups ag,
					" . $this->pref . "acl_options ao
				WHERE
					ug.group_id = ag.group_id AND
					ag.auth_option_id = ao.auth_option_id AND
					ag.auth_setting = 1 AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					au.user_id
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_roles_data ard,
					" . $this->pref . "acl_options ao
				WHERE
					au.auth_role_id = ard.role_id AND
					ard.auth_setting = 1 AND
					ard.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')
				UNION
				SELECT
					au.user_id
				FROM
					" . $this->pref . "acl_users au,
					" . $this->pref . "acl_options ao
				WHERE
					au.auth_setting = 1 AND
					au.auth_option_id = ao.auth_option_id AND
					(ao.auth_option = 'a_cpgadmin' OR ao.auth_option = 'm_cpgadmin')) adm
			ON
				u.user_id = adm.user_id
			LEFT JOIN
				" . $this->pref . "bots b
			ON
				u.user_id = b.user_id "
			. $sql_footer;

			$result = cpg_db_query($sql, $this->link_id);

			// If no records, return empty value
			if (!mysql_num_rows($result))
			{
				return array();
			}

			while ($row = mysql_fetch_assoc($result))
			{
				$userlist[$row['user_id']] = $row;
				$userlist[$row['user_id']]['user_name'] = utf8_encode($userlist[$row['user_id']]['user_name']);
			}
			mysql_free_result($result);

			$user_list_string = implode(', ', array_keys($userlist));

			$sql =
			"SELECT
				owner_id,
				COUNT(pid) as pic_count,
				ROUND(SUM(total_filesize)/1024) as disk_usage
			FROM
				{$C['TABLE_PICTURES']}
			WHERE
				owner_id IN ($user_list_string)
			GROUP BY owner_id";

			$result = cpg_db_query($sql);

			while ($owner = mysql_fetch_assoc($result))
			{
				$userlist[$owner['owner_id']] = array_merge($userlist[$owner['owner_id']], $owner);
			}
			mysql_free_result($result);
		}


		foreach ($userlist as $uid => $user)
		{
			$userlist[$uid]['group_quota'] = $quotas[$user['group_id']];
		}

		foreach ($userlist as $uid => $user)
		{
			if (!isset($user['user_name'])) unset($userlist[$uid]);
		}

		if ($this->adv_sort)
		{
			usort($userlist, array('phpbb_udb', 'adv_sort'));
		}

		return $userlist;
    }

    //---------------------------------------------------------------------------
    function adv_sort($a, $b)
	{
		if ($this->sortdir == 'ASC')
		{
			if ( (int)$a[$this->sortfield] < (int)$b[$this->sortfield] )
				return -1;
			elseif ( (int)$a[$this->sortfield] == (int)$b[$this->sortfield] )
				return 0;
			else
				return 1;
		}
		else
		{
			if ( (int)$b[$this->sortfield] < (int)$a[$this->sortfield] )
				return -1;
			elseif ( (int)$b[$this->sortfield] == (int)$a[$this->sortfield] )
				return 0;
			else
				return 1;
		}
	}


    //---------------------------------------------------------------------------
    function session_update()
    {
		// update once a minute
		if ( time() - $this->session_data['session_time'] > 60 )
		{
			$sql =
				"UPDATE
					" . $this->pref . "sessions
				SET
					 session_time = " . time() . "
				WHERE
					session_id = " . quote_smart($this->sid, $this->link_id);

			cpg_db_query($sql, $this->link_id);
		}
	}
}

// ----------------------------------------------------------------------------------------------------------
// return relative path to 'target_file' up from 'source_file'
// this stuff is from the former version of the bridge
// ----------------------------------------------------------------------------------------------------------
function relative_paths($source_file, $target_file)
{

	$_cpg = dirname(realpath($source_file));
	$_bb = dirname(realpath($target_file));
	$sep = strpos($_cpg, '\\') !== false ? '\\' : '/'; // for IIS

	$_cpg = explode($sep, $_cpg);
	$_bb = explode($sep, $_bb);

	if (count($_cpg) > count($_bb))
	{
		// $_cpg is below $_bb
		$npath = '';
		foreach($_bb as $key=>$path)
		{
		  if ($path == $_cpg[$key])
		    continue;
		  $npath.=$_cpg[$key].'/';
		}
		$npath.=implode('/', array_slice($_cpg, $key+1));
	}
	elseif (count($_bb) > count($_cpg))
	{
		// $_bb is below $cpg
		$npath = '';
		foreach($_cpg as $key=>$path)
		{
		  if ($path == $_bb[$key])
		    continue;
		  $npath.='../';
		}
		$npath.=str_repeat('../', count($_bb) - ($key+1));
	}
	else
	{
		// both on same level
		$npath = '';
		foreach($_cpg as $key=>$path)
		{
		  if ($path == $_bb[$key])
		    continue;
		  $npath.='../'.$_cpg[$key];
		}
	}

	return $npath;
}

// ----------------------------------------------------------------
// Quote variable to make safe - this is against SQL Injections
// ----------------------------------------------------------------
function quote_smart($value, $dblink)
{
	// Stripslashes
	if (get_magic_quotes_gpc())
	{
	    $value = stripslashes($value);
	}

	// Quote if not integer
	if (!is_numeric($value))
	{
		if ( $dblink === 0 )
		{
			$value = "'" . mysql_real_escape_string($value) . "'";
		}
		else
		{
	    	$value = "'" . mysql_real_escape_string($value, $dblink) . "'";
		}
	}
	return $value;
}

// -------------------------------------------------------------------------------
// and go for gold ...
// -------------------------------------------------------------------------------
$cpg_udb = new phpbb3_udb;
?>